==> open-xchange-documents-backend_7.8.2.orig.tar.bz2: open-xchange-documents-backend-7.8.2/com.openexchange.office.odf/src/com/openexchange/office/odt/dom/TextContentHelper.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.odt.dom;

import org.apache.commons.lang.StringUtils;
import org.apache.xml.serializer.SerializationHandler;
import org.odftoolkit.odfdom.IElementWriter;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import com.openexchange.office.odf.DLList;
import com.openexchange.office.odf.INodeAccessor;
import com.openexchange.office.odf.Namespaces;
import com.openexchange.office.odf.SaxContextHandler;

public class TextContentHelper {

    public static SaxContextHandler startElement(SaxContextHandler parentHandler, INodeAccessor contentAccessor, Attributes attributes, String uri, String localName, String qName)
    	throws SAXException {

    	if(qName.equals("text:p")) {
    		final Paragraph paragraph = new Paragraph(false, attributes, null);
    		contentAccessor.getContent().add(paragraph);
    		return new ParagraphHandler(parentHandler, paragraph);
    	}
    	else if(qName.equals("text:h")) {
    		final Paragraph paragraph = new Paragraph(true, attributes, null);
    		contentAccessor.getContent().add(paragraph);
    		return new ParagraphHandler(parentHandler, paragraph);
    	}
    	else if(qName.equals("text:list")) {
    		return new TextListHandler(parentHandler, contentAccessor, new TextList(null, attributes));
    	}
    	else if(qName.equals("table:table")) {
    		final Table table = new Table(attributes);
    		contentAccessor.getContent().add(table);
    		return new TableHandler(parentHandler, table);
    	}
    	return null;
    }

	public static void write(SerializationHandler output, DLList<Object> content)
		throws SAXException {

		TextListItem currentTextListItem = null;
		for(Object child:content) {
			if(child instanceof Paragraph) {
				currentTextListItem = checkAndUpdateTextListLevel(output, currentTextListItem, ((Paragraph)child).getTextListItem());
				if(currentTextListItem!=null) {
					startTextListItem(output, currentTextListItem);
					((Paragraph)child).writeObject(output);
					endTextListItem(output, currentTextListItem);
				}
				else {
					((Paragraph)child).writeObject(output);
				}
			}
			else if (child instanceof IElementWriter){
				currentTextListItem = updateTextListLevel(output, currentTextListItem, null);
				((IElementWriter)child).writeObject(output);
			}
		}
		updateTextListLevel(output, currentTextListItem, null);
	}

	private static TextListItem checkAndUpdateTextListLevel(SerializationHandler output, TextListItem currentTextListItem, TextListItem targetTextListItem)
		throws SAXException {

		// check if both list items have the same parent style
		if(currentTextListItem!=null&&targetTextListItem!=null) {
			final int currentLevel = currentTextListItem.getListLevel();
			final int targetLevel = targetTextListItem.getListLevel();
	
			final int level = currentLevel <= targetLevel ? currentLevel : targetLevel;
			int sameParentLevel = -1;

			TextListItem currentTextListParent = currentTextListItem.getParentTextListItem(level);
			TextListItem targetTextListParent = targetTextListItem.getParentTextListItem(level);

			for(int l=level; l>=0; l--) {
				if(!StringUtils.equals(currentTextListParent.getParentTextList().getStyleName(), targetTextListParent.getParentTextList().getStyleName())) {
					sameParentLevel = l;
				}
				currentTextListParent = currentTextListParent.getParentTextList().getParentTextListItem();
				targetTextListParent = targetTextListParent.getParentTextList().getParentTextListItem();
			}
			if(sameParentLevel!=-1) {
				final int t = sameParentLevel-1;
				if(t==-1) {
					currentTextListItem = updateTextListLevel(output, currentTextListItem, null);
				}
				else {
					currentTextListItem = updateTextListLevel(output, currentTextListItem, targetTextListItem.getParentTextListItem(t));
				}
			}
		}
		return updateTextListLevel(output, currentTextListItem, targetTextListItem);
	}

	private static TextListItem updateTextListLevel(SerializationHandler output, TextListItem currentTextListItem, TextListItem targetTextListItem)
		throws SAXException {

		int currentListLevel = currentTextListItem!=null ? currentTextListItem.getListLevel() : -1;
		int requiredListLevel = targetTextListItem!=null ? targetTextListItem.getListLevel() : -1;

		if(currentListLevel > requiredListLevel) {
			while(currentListLevel > requiredListLevel) {
				currentListLevel--;
				endTextList(output);
				currentTextListItem = currentTextListItem.getParentTextList().getParentTextListItem();
				if(currentTextListItem!=null) {
					endTextListItem(output, currentTextListItem);
				}
			}
		}
		else if(currentListLevel < requiredListLevel) {
			while(currentListLevel < requiredListLevel) {
				currentListLevel++;
				currentTextListItem = targetTextListItem;
				while(currentTextListItem.getListLevel()!=currentListLevel) {
					currentTextListItem = currentTextListItem.getParentTextList().getParentTextListItem();
				}
				final TextListItem parentTextListItem = currentTextListItem.getParentTextList().getParentTextListItem();
				if(parentTextListItem!=null) {
					startTextListItem(output, parentTextListItem);
				}
				startTextList(output, currentTextListItem.getParentTextList());
			}
		}
		return targetTextListItem;
	}

	private static void startTextList(SerializationHandler output, TextList textList)
		throws SAXException {

		output.startElement(Namespaces.TEXT, "list", "text:list");
		textList.writeAttributes(output);
	}

	private static void endTextList(SerializationHandler output)
		throws SAXException {

		output.endElement(Namespaces.TEXT, "list", "text:list");
	}

	private static void startTextListItem(SerializationHandler output, TextListItem textListItem)
		throws SAXException {

		if(textListItem.isHeader()) {
			output.startElement(Namespaces.TEXT, "list-header", "text:list-header");
		}
		else {
			output.startElement(Namespaces.TEXT, "list-item", "text:list-item");
		}
	}

	private static void endTextListItem(SerializationHandler output, TextListItem textListItem)
		throws SAXException {

		if(textListItem.isHeader()) {
			output.endElement(Namespaces.TEXT, "list-header", "text:list-header");
		}
		else {
			output.endElement(Namespaces.TEXT, "list-item", "text:list-item");
		}
	}
}

==> open-xchange-documents-backend_7.8.3.orig.tar.bz2: open-xchange-documents-backend-7.8.3/com.openexchange.office.odf/src/com/openexchange/office/odt/dom/TextContentHelper.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.odt.dom;

import java.util.Stack;
import org.apache.commons.lang.StringUtils;
import org.apache.xml.serializer.SerializationHandler;
import org.odftoolkit.odfdom.IElementWriter;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import com.openexchange.office.odf.DLList;
import com.openexchange.office.odf.INodeAccessor;
import com.openexchange.office.odf.Namespaces;
import com.openexchange.office.odf.SaxContextHandler;

public class TextContentHelper {

    public static SaxContextHandler startElement(SaxContextHandler parentHandler, INodeAccessor contentAccessor, Attributes attributes, String uri, String localName, String qName)
    	throws SAXException {

    	if(qName.equals("text:p")) {
    		final Paragraph paragraph = new Paragraph(false, attributes, null);
    		contentAccessor.getContent().add(paragraph);
    		return new ParagraphHandler(parentHandler, paragraph);
    	}
    	else if(qName.equals("text:h")) {
    		final Paragraph paragraph = new Paragraph(true, attributes, null);
    		contentAccessor.getContent().add(paragraph);
    		return new ParagraphHandler(parentHandler, paragraph);
    	}
    	else if(qName.equals("text:list")) {
    		return new TextListHandler(parentHandler, contentAccessor, new TextList(null, attributes));
    	}
    	else if(qName.equals("table:table")) {
    		final Table table = new Table(attributes);
    		contentAccessor.getContent().add(table);
    		return new TableHandler(parentHandler, table);
    	}
    	return null;
    }

	public static void write(SerializationHandler output, DLList<Object> content)
		throws SAXException {

	    final Stack<Boolean> listHeaderStack = new Stack<Boolean>();
		TextListItem currentTextListItem = null;
		for(Object child:content) {
			if(child instanceof Paragraph) {
				currentTextListItem = checkAndUpdateTextListLevel(output, currentTextListItem, ((Paragraph)child).getTextListItem(), listHeaderStack);
				((Paragraph)child).writeObject(output);
			}
			else if (child instanceof ISectionWriter) {
				currentTextListItem = updateTextListLevel(output, currentTextListItem, null, listHeaderStack);
				((ISectionWriter)child).writeSection(output, content);
			}
			else if (child instanceof IElementWriter){
				currentTextListItem = updateTextListLevel(output, currentTextListItem, null, listHeaderStack);
				((IElementWriter)child).writeObject(output);
			}
		}
		updateTextListLevel(output, currentTextListItem, null, listHeaderStack);
	}

	private static TextListItem checkAndUpdateTextListLevel(SerializationHandler output, TextListItem currentTextListItem, TextListItem targetTextListItem, Stack<Boolean> listHeaderStack)
		throws SAXException {

		// check if both list items have the same parent style
		if(currentTextListItem!=null&&targetTextListItem!=null) {
			final int currentLevel = currentTextListItem.getListLevel();
			final int targetLevel = targetTextListItem.getListLevel();
	
			final int level = currentLevel <= targetLevel ? currentLevel : targetLevel;
			int sameParentLevel = -1;

			TextListItem currentTextListParent = currentTextListItem.getParentTextListItem(level);
			TextListItem targetTextListParent = targetTextListItem.getParentTextListItem(level);

			for(int l=level; l>=0; l--) {
				if(StringUtils.equals(currentTextListParent.getParentTextList().getStyleName(false), targetTextListParent.getParentTextList().getStyleName(false))) {
					sameParentLevel = l;
					break;
				}
				currentTextListParent = currentTextListParent.getParentTextList().getParentTextListItem();
				targetTextListParent = targetTextListParent.getParentTextList().getParentTextListItem();
			}
			if(sameParentLevel==-1) {
				currentTextListItem = updateTextListLevel(output, currentTextListItem, null, listHeaderStack);
			}
			else {
				currentTextListItem = updateTextListLevel(output, currentTextListItem, targetTextListItem.getParentTextListItem(sameParentLevel), listHeaderStack);
			}
		}
		return updateTextListLevel(output, currentTextListItem, targetTextListItem, listHeaderStack);
	}

	private static TextListItem updateTextListLevel(SerializationHandler output, TextListItem currentTextListItem, TextListItem targetTextListItem, Stack<Boolean> listHeaderStack)
		throws SAXException {

		int currentListLevel = currentTextListItem!=null ? currentTextListItem.getListLevel() : -1;
		int requiredListLevel = targetTextListItem!=null ? targetTextListItem.getListLevel() : -1;

		if(currentListLevel==requiredListLevel) {
			if(currentListLevel>-1) {
				endTextListItem(output, listHeaderStack);
				startTextListItem(output, targetTextListItem, listHeaderStack);
			}
		}
		else if(currentListLevel > requiredListLevel) {
			while(currentListLevel > requiredListLevel) {
				currentListLevel--;
				endTextListItem(output, listHeaderStack);
				endTextList(output);
				currentTextListItem = currentTextListItem.getParentTextList().getParentTextListItem();
			}
		}
		else {
			while(currentListLevel < requiredListLevel) {
				currentListLevel++;
				currentTextListItem = targetTextListItem;
				while(currentTextListItem.getListLevel()!=currentListLevel) {
					currentTextListItem = currentTextListItem.getParentTextList().getParentTextListItem();
				}
				startTextList(output, currentTextListItem.getParentTextList());
				startTextListItem(output, currentTextListItem, listHeaderStack);
			}
		}
		return targetTextListItem;
	}

	private static void startTextList(SerializationHandler output, TextList textList)
		throws SAXException {

		output.startElement(Namespaces.TEXT, "list", "text:list");
		textList.writeAttributes(output);
	}

	private static void endTextList(SerializationHandler output)
		throws SAXException {

		output.endElement(Namespaces.TEXT, "list", "text:list");
	}

	private static void startTextListItem(SerializationHandler output, TextListItem textListItem, Stack<Boolean> listHeaderStack)
		throws SAXException {

	    final boolean isHeader = textListItem.isHeader();
	    listHeaderStack.push(Boolean.valueOf(isHeader));
		if(isHeader) {
			output.startElement(Namespaces.TEXT, "list-header", "text:list-header");
		}
		else {
			output.startElement(Namespaces.TEXT, "list-item", "text:list-item");
		}
	}

	private static void endTextListItem(SerializationHandler output, Stack<Boolean> listHeaderStack)
		throws SAXException {

	    final boolean isHeader = listHeaderStack.pop();
		if(isHeader) {
			output.endElement(Namespaces.TEXT, "list-header", "text:list-header");
		}
		else {
			output.endElement(Namespaces.TEXT, "list-item", "text:list-item");
		}
	}
}
